package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.base.service.IBaseWeatherCloudService;
import pps.cloud.middle.service.IWeatherAttachmentCloudService;
import pps.cloud.middle.service.data.weather_attachment_record.GetWeatherAttachmentRecordInput;
import pps.cloud.middle.service.data.weather_attachment_record.GetWeatherAttachmentRecordOutput;
import pps.core.base.entity.BasePhotovoltaicPlantView;
import pps.core.base.entity.PlantPredictedPowerDataEnt;
import pps.core.base.entity.WeatherBureauDataView;
import pps.core.base.enums.WeatherIconEnum;
import pps.core.base.enums.WindDirection;
import pps.core.base.mapper.BasePhotovoltaicPlantViewMapper;
import pps.core.base.mapper.PlantPredictedPowerDataMapper;
import pps.core.base.mapper.PlantPredictedPowerDataViewMapper;
import pps.core.base.mapper.WeatherBureauDataViewMapper;
import pps.core.base.service.data.third_weather_data.DailyData;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.utils.BaseUtils;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.XStartup;
import xstartup.base.tool.XLoggerTool;
import xstartup.base.util.XCopyUtils;
import xstartup.core.base.helper.XThreadHelper;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;

import java.io.File;
import java.math.BigDecimal;
import java.time.LocalTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 天气数据接收Cloud模块
 *
 * @author ZWT
 * @date 2023/11/13 17:54
 */
@XService
public class BaseWeatherCloudServiceImpl implements IBaseWeatherCloudService {

    private static final String storeDir = XStartup.getCurrent().getProperty("weather.file.temp.path");

    /**
     * 天气数据接收Cloud模块--天气数据处理
     *
     * @param context      上下文
     * @param oilFieldCode 油田编码
     * @return {@link XServiceResult }
     */
    @Override
    public XServiceResult weatherDataProcess(XContext context, String oilFieldCode) {
        XLoggerTool logger = context.getLogger();
        List<JSONObject> jsonObjectList = new ArrayList<>(48);
        logger.info("------ weatherDataProcess select mail ------");
        IWeatherAttachmentCloudService service = context.getBean(IWeatherAttachmentCloudService.class);
        XSingleResult<GetWeatherAttachmentRecordOutput> result = service.getEmail(context, new GetWeatherAttachmentRecordInput());
        result.throwIfFail();
        GetWeatherAttachmentRecordOutput output = result.getResult();
        if (ObjectUtil.isNotNull(output) && CharSequenceUtil.isAllNotBlank(output.getEmailName(), output.getEmailAttachmentUrl())) {
            String filePath = this.createFilePath(output.getEmailName());
            logger.info("------ weatherDataProcess load file: {}", filePath);
            HttpUtil.downloadFile(output.getEmailAttachmentUrl(), filePath);
            JSONObject jsonObject = JSON.parseObject(FileUtil.readUtf8String(filePath));
            jsonObjectList.add(jsonObject);
            FileUtil.del(filePath);
        }
        logger.info("------ weatherDataProcess jsonObjectList size: {}", jsonObjectList.size());
        if (CollUtil.isNotEmpty(jsonObjectList)) {
            //气象局数据处理
            XThreadHelper.async(() -> {
                this.weatherBureauDataProcess(context, jsonObjectList);
            });
            //光伏预测数据处理
            XThreadHelper.async(() -> {
                this.photovoltaicDataProcess(context, jsonObjectList);
                //预测接口
                XThreadHelper.async(() -> {
                    try {
                        logger.info("------ weatherDataProcess predict");
                        HttpUtil.get("http://127.0.0.1:10060/api/photovoltaic/predict?secretKey=fZ1kC1aV8bO7rH3bL8oE5zI0mY3vW1sG");
                    } catch (Exception e) {
                        context.getLogger().error("---- predict", e);
                    }
                });
            });
        }
        return XServiceResult.OK;
    }

    /*------------------------- private -------------------------*/

    /**
     * 计算
     *
     * @param a 一
     * @param b b
     * @param c c
     * @return {@link BigDecimal}
     */
    private BigDecimal compute(BigDecimal a, BigDecimal b, BigDecimal c) {
        return a.add(b.subtract(a).multiply(c));
    }

    /**
     * int转String
     *
     * @param num 数字
     * @return {@link String}
     */
    private String int2Str(int num) {
        return num < 10 ? "0" + num : String.valueOf(num);
    }

    /**
     * 创建文件路径
     *
     * @param fileName 文件名
     * @return {@link String }
     */
    private String createFilePath(String fileName) {
        StringBuilder stringBuilder = new StringBuilder(storeDir)
                .append(File.separator)
                .append(DateTime.now().toString("yyyyMMdd"));
        return stringBuilder.append(File.separator).append(fileName).toString();
    }


    /**
     * 气象局数据处理
     *
     * @param context        上下文
     * @param jsonObjectList json对象列表
     */
    private void weatherBureauDataProcess(XContext context, List<JSONObject> jsonObjectList) {
        context.getLogger().info("------ weatherDataProcess weatherBureauDataProcess begin ------");
        List<String> deleteList = new ArrayList<>(32);
        List<WeatherBureauDataView> batchList = new ArrayList<>(BusinessConstant.LIST_MAX_SIZE);
        DateTime create = DateUtil.date();
        DateTime end = DateUtil.offsetDay(create, 16);
        int count = 0;
        for (JSONObject jsonObject : jsonObjectList) {
            this.dataFractionation(jsonObject, deleteList, batchList);
            if (CollUtil.isEmpty(batchList)) {
                continue;
            }
            if (count == 0) {
                batchList.sort(Comparator.comparing(WeatherBureauDataView::getDataTime));
                create = DateUtil.offsetMinute(batchList.get(0).getDataTime(), -5);
            }
            count++;
            if (BusinessConstant.TEN.equals(count)) {
                this.deleteWeatherBureauList(context, deleteList, create, end);
                this.insertWeatherBureauList(context, batchList);
                context.getLogger().info("------ weatherDataProcess clear size d {} b {}", deleteList.size(), batchList.size());
                CollUtil.clear(deleteList, batchList);
                count = 0;
            }
        }
        context.getLogger().info("------ weatherDataProcess delete and insertBatch");
        if (CollUtil.isNotEmpty(deleteList)) {
            this.deleteWeatherBureauList(context, deleteList, create, end);
        }
        if (CollUtil.isNotEmpty(batchList)) {
            this.insertWeatherBureauList(context, batchList);
        }
        CollUtil.clear(deleteList, batchList);
        context.getLogger().info("------ weatherDataProcess weatherBureauDataProcess end ------");
    }

    /**
     * 数据处理
     *
     * @param jsonObject json对象
     * @param deleteList 删除列表
     * @param batchList  批次列表
     */
    private void dataFractionation(JSONObject jsonObject, List<String> deleteList, List<WeatherBureauDataView> batchList) {
        Map<String, List<DailyData>> cityDataMap = new HashMap<>(16);
        for (Map.Entry<String, Object> entry : jsonObject.entrySet()) {
            String key = entry.getKey();
            Object value = entry.getValue();
            if (value instanceof JSONArray) {
                cityDataMap.put(key, ((JSONArray) value).toJavaList(DailyData.class));
            }
        }
        for (Map.Entry<String, List<DailyData>> entry : cityDataMap.entrySet()) {
            List<DailyData> dailyDataList = entry.getValue();
            String cityCode = entry.getKey();
            //删除数据
            deleteList.add(cityCode);
            //遍历预测数据列表
            int flag = 0;
            DateTime dateTime;
            for (DailyData dailyData : dailyDataList) {
                dateTime = DateUtil.date(dailyData.getDateTime());
                for (int i = 0; i < 8; i++) {
                    int hour = LocalTime.parse(dailyData.getHourTimeArray().get(i)).getHour();
                    DateTime parse = DateUtil.parse(String.format(
                            "%s %s:%s",
                            DateUtil.formatDate(dateTime),
                            this.int2Str(hour),
                            BusinessConstant.START_OF_DAY_TIME
                    ));
                    if (DateUtil.compare(parse, dateTime) < 0 && flag == 0) {
                        parse = DateUtil.offsetDay(parse, 1);
                        dateTime = DateUtil.offsetDay(dateTime, 1);
                        flag++;
                    }
                    WeatherBureauDataView view = WeatherBureauDataView.builder()
                            .dataTime(parse)
                            .weatherIcon(this.getStringByIndex(dailyData.getWeatherArray(), i))
                            .airTemperature(this.stringToBigDecimal(dailyData.getTemperatureArray(), i, "℃"))
                            .precipitation(this.getStringByIndex(dailyData.getPrecipitationArray(), i))
                            .windSpeed(this.stringToBigDecimal(dailyData.getWindSpeedArray(), i, "m/s"))
                            .windDirection(this.getWindDirection(dailyData.getWindDirectionArray(), i))
                            .pressure(this.stringToBigDecimal(dailyData.getPressureArray(), i, "hPa"))
                            .humidity(this.stringToBigDecimal(dailyData.getHumidityArray(), i, "%"))
                            .cloudCover(this.stringToBigDecimal(dailyData.getCloudCoverArray(), i, "%"))
                            .areaCode(cityCode)
                            .build();
                    //去重
                    if (batchList.stream()
                            .noneMatch(item -> DateUtil.compare(item.getDataTime(), view.getDataTime()) == 0 && CharSequenceUtil.equals(item.getAreaCode(), view.getAreaCode()))) {
                        batchList.add(view);
                    }
                }
            }
        }
    }

    /**
     * 条件删除预测数据
     *
     * @param context    上下文
     * @param deleteList 删除列表
     * @param create     创造
     * @param end        结束
     */
    private void deleteWeatherBureauList(XContext context, List<String> deleteList, DateTime create, DateTime end) {
        if (CollUtil.isNotEmpty(deleteList)) {
            WeatherBureauDataViewMapper mapper = context.getBean(WeatherBureauDataViewMapper.class);
            int size = deleteList.size();
            context.getLogger().info("------ deleteWeatherBureauList batchDelete delete size: {}", size);
            if (size > BaseUtils.BATCH_SIZE) {
                List<List<String>> subList = BaseUtils.getSubList(deleteList);
                for (List<String> list : subList) {
                    mapper.deleteBatch(list, create, end);
                }
            } else {
                mapper.deleteBatch(deleteList, create, end);
            }
        }
    }

    /**
     * 插入间歇装置预测功率列表
     *
     * @param context   上下文
     * @param batchList 批处理列表
     */
    private void insertWeatherBureauList(XContext context, List<WeatherBureauDataView> batchList) {
        if (CollUtil.isNotEmpty(batchList)) {
            WeatherBureauDataViewMapper dataViewMapper = context.getBean(WeatherBureauDataViewMapper.class);
            int size = batchList.size();
            context.getLogger().info("------ insertWindTurbineList batchList insert size: {}", size);
            if (size > BaseUtils.BATCH_SIZE) {
                List<List<WeatherBureauDataView>> subList = BaseUtils.getSubList(batchList);
                subList.forEach(dataViewMapper::insertBatch);
            } else {
                dataViewMapper.insertBatch(batchList);
            }
        }
    }

    /**
     * 光伏数据处理
     *
     * @param context        上下文
     * @param jsonObjectList json对象列表
     */
    private void photovoltaicDataProcess(XContext context, List<JSONObject> jsonObjectList) {
        context.getLogger().info("------ weatherDataProcess photovoltaicDataProcess begin ------");
        //取电站(排除第二采油厂)
        BasePhotovoltaicPlantViewMapper viewMapper = context.getBean(BasePhotovoltaicPlantViewMapper.class);
        List<BasePhotovoltaicPlantView> plantList = viewMapper.selectPlantList(new BasePhotovoltaicPlantView());
        if (CollUtil.isEmpty(plantList)) {
            context.getLogger().info("------ weatherDataProcess plantList is empty ------");
            return;
        }
        context.getLogger().info("------ weatherDataProcess dataFractionation");
        List<String> deleteList = new ArrayList<>(32);
        List<PlantPredictedPowerDataEnt> batchList = new ArrayList<>(BusinessConstant.LIST_MAX_SIZE);
        DateTime create = DateUtil.date();
        DateTime end = DateUtil.offsetDay(create, 16);
        //按照cityCode分组
        Map<String, List<BasePhotovoltaicPlantView>> plantMap = plantList.stream()
                .collect(Collectors.groupingBy(BasePhotovoltaicPlantView::getCityCode));
        int count = 0;
        Map<String, Set<String>> temp = new HashMap<>(32);
        for (JSONObject jsonObject : jsonObjectList) {
            this.dataFractionation(jsonObject, plantMap, deleteList, batchList, temp);
            if (CollUtil.isEmpty(batchList)) {
                continue;
            }
            //分钟转换
            batchList.forEach(b -> b.setMinTime(CharSequenceUtil.replaceLast(b.getMinTime(), ":00", "")));
            if (count == 0) {
                batchList.sort(Comparator.comparing(PlantPredictedPowerDataEnt::getDataDate));
                create = DateUtil.offsetMinute(DateUtil.parse(batchList.get(0).getDataDate()), -5);
            }
            count++;
            if (BusinessConstant.FIVE.equals(count)) {
                this.deletePlantPredictedPowerList(context, deleteList, create, end);
                this.insertBatchPlantPredictedPowerList(context, batchList);
                context.getLogger().info("------ weatherDataProcess clear size d {} b {}", deleteList.size(), batchList.size());
                CollUtil.clear(deleteList, batchList);
                count = 0;
            }
        }
        context.getLogger().info("------ weatherDataProcess delete and insertBatch");
        if (CollUtil.isNotEmpty(deleteList)) {
            this.deletePlantPredictedPowerList(context, deleteList, create, end);
        }
        if (CollUtil.isNotEmpty(batchList)) {
            this.insertBatchPlantPredictedPowerList(context, batchList);
        }
        CollUtil.clear(deleteList, batchList);
        context.getLogger().info("------ weatherDataProcess photovoltaicDataProcess end ------");
    }

    /**
     * 数据拆分
     *
     * @param jsonObject json对象
     * @param plantMap   植物地图
     * @param deleteList 删除列表
     * @param batchList  批次列表
     */
    private void dataFractionation(JSONObject jsonObject, Map<String, List<BasePhotovoltaicPlantView>> plantMap, List<String> deleteList, List<PlantPredictedPowerDataEnt> batchList, Map<String, Set<String>> temp) {
        Map<String, List<DailyData>> cityDataMap = new HashMap<>(16);
        for (Map.Entry<String, Object> entry : jsonObject.entrySet()) {
            String key = entry.getKey();
            Object value = entry.getValue();
            if (value instanceof JSONArray) {
                cityDataMap.put(key, ((JSONArray) value).toJavaList(DailyData.class));
            }
        }
        Set<String> timeSet;
        String cityCode;
        for (Map.Entry<String, List<DailyData>> entry : cityDataMap.entrySet()) {
            cityCode = entry.getKey();
            if (!plantMap.containsKey(cityCode)) {
                continue;
            }
            List<DailyData> dailyDataList = entry.getValue();
            List<BasePhotovoltaicPlantView> plantViewList = plantMap.get(cityCode);
            //删除数据
            for (BasePhotovoltaicPlantView plantView : plantViewList) {
                deleteList.add(plantView.getId());
            }
            //遍历预测数据列表
            for (DailyData dailyData : dailyDataList) {
                for (BasePhotovoltaicPlantView plant : plantViewList) {
                    DateTime dateTime = DateUtil.date(dailyData.getDateTime());
                    List<PlantPredictedPowerDataEnt> allList = new ArrayList<>(32);
                    List<PlantPredictedPowerDataEnt> objList = new ArrayList<>(16);
                    String plantId = plant.getId();
                    String stationName = plant.getStationName();
                    //获取时间临时缓存
                    if (temp.containsKey(plantId)) {
                        timeSet = temp.get(plantId);
                    } else {
                        timeSet = new HashSet<>(128);
                        temp.put(plantId, timeSet);
                    }
                    DateTime date;
                    boolean nextDay = false;
                    for (int i = 0; i < 8; i++) {
                        if (nextDay) {
                            dateTime = DateUtil.offsetDay(dateTime, 1);
                        }
                        //判断小时数与查到的小时数，如果查到的小时数小，判断为第二天
                        int hour = LocalTime.parse(dailyData.getHourTimeArray().get(i)).getHour();
                        DateTime parse = DateUtil.parse(String.format(
                                "%s %s:%s",
                                DateUtil.formatDate(dateTime),
                                this.int2Str(hour),
                                BusinessConstant.START_OF_DAY_TIME
                        ));
                        if (hour == 23) {
                            nextDay = true;
                        } else {
                            nextDay = false;
                        }
                        PlantPredictedPowerDataEnt dao = this.getPlantPredictedPowerEnt(plantId, stationName, cityCode,
                                this.int2Str(hour),
                                this.stringToBigDecimal(dailyData.getTemperatureArray(), i, "℃"),
                                this.stringToBigDecimal(dailyData.getWindSpeedArray(), i, "m/s"),
                                new BigDecimal(this.getWindDirection(dailyData.getWindDirectionArray(), i)),
                                this.stringToBigDecimal(dailyData.getPressureArray(), i, "hPa"),
                                this.stringToBigDecimal(dailyData.getHumidityArray(), i, "%"),
                                String.valueOf(parse.year()),
                                this.int2Str(parse.monthBaseOne()),
                                this.int2Str(parse.dayOfMonth()),
                                this.getWeatherType(dailyData.getWeatherArray(), i),
                                this.getPrecipitation(dailyData.getPrecipitationArray(), i)
                        );
                        // 5.执行SQL
                        allList.add(dao);
                        PlantPredictedPowerDataEnt isExit = objList.stream()
                                .filter(item ->
                                        CharSequenceUtil.equals(item.getYearTime(), dao.getYearTime()) &&
                                                CharSequenceUtil.equals(item.getMonthTime(), dao.getMonthTime()) &&
                                                CharSequenceUtil.equals(item.getDayTime(), dao.getDayTime()) &&
                                                CharSequenceUtil.equals(item.getHourTime(), dao.getHourTime()))
                                .findFirst()
                                .orElse(null);
                        if (Objects.isNull(isExit)) {
                            objList.add(dao);
                        }
                    }
                    // 原站点时间结构为：08:00	11:00	14:00	17:00	20:00	23:00	02:00	05:00
                    // 两条数据中间差了两个小时，使用上下两条数据推算出中间差掉的两个小时
                    for (int i = 0; i < objList.size() - 1; i++) {
                        PlantPredictedPowerDataEnt dataEnt = objList.get(i);
                        date = DateTime.of(dataEnt.getDataDate(), "yyyy-MM-dd HH:mm");
                        String hourString;
                        Integer hourTime = Integer.valueOf(dataEnt.getHourTime());
                        if (hourTime.equals(23)) {
                            date = DateUtil.offsetDay(date, 1);
                        }
                        for (int y = 1; y < 3; y++) {
                            if (hourTime.equals(23)) {
                                hourString = "0" + (y - 1);
                            } else {
                                if ((hourTime + y) < 10) {
                                    hourString = "0" + (hourTime + y);
                                } else {
                                    hourString = String.valueOf(hourTime + y);
                                }
                            }
                            PlantPredictedPowerDataEnt nextData = objList.get(i + 1);
                            BigDecimal valueOf = BigDecimal.valueOf(0.3 * y);
                            PlantPredictedPowerDataEnt dao = this.getPlantPredictedPowerEnt(plantId, stationName,
                                    cityCode, hourString,
                                    this.compute(dataEnt.getTemperature(), nextData.getTemperature(), valueOf),
                                    this.compute(dataEnt.getWindSpeed(), nextData.getWindSpeed(), valueOf),
                                    dataEnt.getWindDirection(),
                                    this.compute(dataEnt.getPressure(), nextData.getPressure(), valueOf),
                                    this.compute(dataEnt.getHumidity(), nextData.getHumidity(), valueOf),
                                    String.valueOf(date.year()),
                                    this.int2Str(date.month() + 1),
                                    this.int2Str(date.dayOfMonth()),
                                    dataEnt.getWeatherType(),
                                    this.compute(dataEnt.getPrecipitation(), nextData.getPrecipitation(), valueOf)
                            );
                            // 5.执行SQL
                            allList.add(dao);
                        }
                    }
                    //入库
                    allList.sort(Comparator.comparing(PlantPredictedPowerDataEnt::getDataDate));
                    for (int i = 0; i < allList.size(); i++) {
                        PlantPredictedPowerDataEnt item = allList.get(i);
                        //判断时间是否存在
                        if (timeSet.contains(item.getDataDate())) {
                            continue;
                        } else {
                            timeSet.add(item.getDataDate());
                        }
                        for (int y = 0; y < BusinessConstant.TIME_LIST.size(); y++) {
                            PlantPredictedPowerDataEnt dao = XCopyUtils.copyNewObject(item);
                            dao.setMinTime(BusinessConstant.TIME_LIST.get(y));
                            dao.setDataDate(dao.getYearTime() + '-' + dao.getMonthTime() + '-' + dao.getDayTime() + ' ' + dao.getHourTime() + ':' + dao.getMinTime());
                            batchList.add(dao);
                        }
                        batchList.add(item);
                    }
                }
            }
        }
    }

    /**
     * 获取电厂预测功率ent
     *
     * @param plantId       电站ID
     * @param stationName   电站名称
     * @param cityCode      地区编码
     * @param hourTime      小时
     * @param temperature   温度
     * @param windSpeed     风速
     * @param windDirection 风向
     * @param pressure      压力
     * @param humidity      湿度
     * @param yearTime      年时间
     * @param monthTime     月份时间
     * @param dayTime       白天
     * @param weatherType   天气类型
     * @param precipitation 降水量
     * @return {@link PlantPredictedPowerDataEnt}
     */
    private PlantPredictedPowerDataEnt getPlantPredictedPowerEnt(String plantId, String stationName, String cityCode, String hourTime, BigDecimal temperature, BigDecimal windSpeed, BigDecimal windDirection,
                                                                 BigDecimal pressure, BigDecimal humidity, String yearTime, String monthTime, String dayTime, String weatherType, BigDecimal precipitation) {
        PlantPredictedPowerDataEnt dao = new PlantPredictedPowerDataEnt();
        dao.setPlantName(stationName);
        dao.setAreaCode(cityCode);
        dao.setCreateTime(DateUtil.date());
        dao.setPlantId(plantId);
        dao.setHourTime(hourTime);
        dao.setTemperature(temperature);
        dao.setWindSpeed(windSpeed);
        dao.setWindDirection(windDirection);
        dao.setPressure(pressure);
        dao.setHumidity(humidity);
        dao.setYearTime(yearTime);
        dao.setMonthTime(monthTime);
        dao.setDayTime(dayTime);
        dao.setMinTime(BusinessConstant.START_OF_DAY_TIME);
        dao.setDataDate(dao.getYearTime() + '-' + dao.getMonthTime() + '-' + dao.getDayTime() + ' ' + dao.getHourTime() + ':' + dao.getMinTime());
        dao.setWeatherType(weatherType);
        dao.setPrecipitation(precipitation);
        return dao;
    }

    /**
     * 条件删除预测数据
     *
     * @param context    上下文
     * @param deleteList 删除列表
     * @param create     创造
     * @param end        结束
     */
    private void deletePlantPredictedPowerList(XContext context, List<String> deleteList, DateTime create, DateTime end) {
        if (CollUtil.isNotEmpty(deleteList)) {
            PlantPredictedPowerDataMapper mapper = context.getBean(PlantPredictedPowerDataMapper.class);
            int size = deleteList.size();
            context.getLogger().info("------ deletePlantPredictedPowerList batchDelete delete size: {}", size);
            if (size > BaseUtils.BATCH_SIZE) {
                List<List<String>> subList = BaseUtils.getSubList(deleteList);
                for (List<String> list : subList) {
                    mapper.delete(new LambdaQueryWrapper<PlantPredictedPowerDataEnt>()
                            .in(PlantPredictedPowerDataEnt::getPlantId, list)
                            .between(PlantPredictedPowerDataEnt::getDataDate, create, end));
                }
            } else {
                mapper.delete(new LambdaQueryWrapper<PlantPredictedPowerDataEnt>()
                        .in(PlantPredictedPowerDataEnt::getPlantId, deleteList)
                        .between(PlantPredictedPowerDataEnt::getDataDate, create, end));
            }
        }
    }

    /**
     * 插入间歇装置预测功率列表
     *
     * @param context   上下文
     * @param batchList 批处理列表
     */
    private void insertBatchPlantPredictedPowerList(XContext context, List<PlantPredictedPowerDataEnt> batchList) {
        if (CollUtil.isNotEmpty(batchList)) {
            PlantPredictedPowerDataViewMapper dataViewMapper = context.getBean(PlantPredictedPowerDataViewMapper.class);
            int size = batchList.size();
            context.getLogger().info("------ insertBatchPlantPredictedPowerList batchList insert size: {}", size);
            if (size > BaseUtils.BATCH_SIZE) {
                List<List<PlantPredictedPowerDataEnt>> subList = BaseUtils.getSubList(batchList);
                subList.forEach(dataViewMapper::insertBatch);
            } else {
                dataViewMapper.insertBatch(batchList);
            }
        }
    }

    /**
     * 获取风向
     *
     * @param windDirectionArray 风向阵列
     * @param i                  我
     * @return int
     */
    private int getWindDirection(List<String> windDirectionArray, int i) {
        int wind = 0;
        if (CollUtil.isNotEmpty(windDirectionArray)) {
            if (i < windDirectionArray.size()) {
                WindDirection windDirection = WindDirection.findByValue(windDirectionArray.get(i));
                if (ObjectUtil.isNotNull(windDirection)) {
                    wind = windDirection.getValue();
                }
            }
        }
        return wind;
    }

    /**
     * 字符串转换为BigDecimal
     *
     * @param stringList 字符串列表
     * @param i          我
     * @param replaceStr 替换str
     * @return {@link BigDecimal }
     */
    private BigDecimal stringToBigDecimal(List<String> stringList, int i, String replaceStr) {
        BigDecimal result;
        if (CollUtil.isNotEmpty(stringList) && i < stringList.size() && CharSequenceUtil.isNotBlank(stringList.get(i))) {
            result = new BigDecimal(CharSequenceUtil.replace(stringList.get(i), replaceStr, ""));
        } else {
            result = BigDecimal.ZERO;
        }
        return result;
    }

    /**
     * 获取天气类型
     *
     * @param stringList 字符串列表
     * @param i          我
     * @return {@link String }
     */
    private String getWeatherType(List<String> stringList, int i) {
        String result;
        if (CollUtil.isNotEmpty(stringList) && i < stringList.size() && CharSequenceUtil.isNotBlank(stringList.get(i))) {
            result = WeatherIconEnum.findByValue(stringList.get(i)).getType().toString();
        } else {
            result = "99";
        }
        return result;
    }

    /**
     * 获得降水
     *
     * @param stringList 字符串列表
     * @param i          我
     * @return {@link BigDecimal }
     */
    private BigDecimal getPrecipitation(List<String> stringList, int i) {
        BigDecimal result;
        if (CollUtil.isNotEmpty(stringList) && i < stringList.size() &&
                CharSequenceUtil.isNotBlank(stringList.get(i)) && !CharSequenceUtil.equals(stringList.get(i), "无降水")) {
            result = new BigDecimal(CharSequenceUtil.replace(stringList.get(i), "mm", ""));
        } else {
            result = BigDecimal.ZERO;
        }
        return result;
    }

    /**
     * 按索引位置取字符串
     *
     * @param stringList 字符串列表
     * @param i          我
     * @return {@link String }
     */
    private String getStringByIndex(List<String> stringList, int i) {
        String result;
        if (CollUtil.isNotEmpty(stringList) && i < stringList.size() && CharSequenceUtil.isNotBlank(stringList.get(i))) {
            result = stringList.get(i);
        } else {
            result = BusinessConstant.DEFAULT_VALUE;
        }
        return result;
    }
}